<?php
// ---------------------------------------------------------------
// KwsPHP kOoGar Web System
// $Id: class.mail.php 1049 2017-07-11 16:16:33Z mike007 $
// ---------------------------------------------------------------
// Fichier  : class_mail.php
// Cration : Otocinclus avril 2009
// ---------------------------------------------------------------

// Selon les versions, la fonction php mime_content_type() n'est pas toujours disponible.
// Si elle n'existe pas, dfinition d'une fonction de remplacement
if (!function_exists('mime_content_type'))
{
	/**
	 * Renvoi le type MIME d'un fichier
	 *
	 * @param	string	$filmane	Nom du fichier
	 * @return	string
	 */
	function mime_content_type($filename)
	{
		// Type MIME les plus courants. Il est possible d'en ajouter si besoin.
		static $mime = array(
			'.txt'	=> 'text/plain',
			'.html'	=> 'text/html',
			'.htm'	=> 'text/html',
			'.xml'	=> 'text/xml',
			'.css'	=> 'text/css',
			'.js'	=> 'text/javascript',
			'.bmp'	=> 'image/bmp',
			'.gif'	=> 'image/gif',
			'.jpeg'	=> 'image/jpeg',
			'.jpg'	=> 'image/jpeg',
			'.png'	=> 'image/png',
			'.tiff'	=> 'image/tiff',
			'.tif'	=> 'image/tiff',
			'.ico'	=> 'image/x-icon',
			'.xls'	=> 'application/excel',
			'.doc'	=> 'application/msword',
			'.dot'	=> 'application/msword',
			'.ppt'	=> 'application/powerpoint',
			'.pps'	=> 'application/powerpoint',
			'.pdf'	=> 'application/pdf',
			'.rm'	=> 'application/vnd.rn-realmedia',
			'.gtar'	=> 'application/x-gtar',
			'.tgz'	=> 'application/x-gtar',
			'.swf'	=> 'application/x-shockwave-flash',
			'.tar'	=> 'application/x-tar',
			'.gz'	=> 'application/x-gzip-compressed',
			'.zip'	=> 'application/x-zip-compressed',
			'.xhtml'=> 'application/xhtml+xml',
			'.exe'	=> 'application/octet-stream',
			'.au'	=> 'audio/basic',
			'.snd'	=> 'audio/basic',
			'.aif'	=> 'audio/x-aiff',
			'.aiff'	=> 'audio/x-aiff',
			'.wma'	=> 'audio/x-ms-wma',
			'.mp3'	=> 'audio/mpeg',
			'.wav'	=> 'audio/x-wav',
			'.mpeg'	=> 'video/mpeg',
			'.mpg'	=> 'video/mpeg',
			'.mov'	=> 'video/quicktime',
			'.qt'	=> 'video/quicktime',
			'.avi'	=> 'video/x-msvideo',
			'.movie'=> 'video/x-sgi-movie'
		);

		$ext = strrchr($filename, '.');
		$mime_type = $mime[$ext];
		if (empty($mime_type))
		{
			$mime_type = 'application/octet-stream';
		}

		return $mime_type;
	}
}

define('MAIL_TXT'  , 0);
define('MAIL_HTML' , 1);
define('MAIL_MULTI', 2);
define('MAIL_BCC', 0);
define('MAIL_TO1', 1);
define('MAIL_TO2', 2);

/**
 * Classe grant les mails
 *
 * Gre l'envoi de mails au format txt, html ou multi-formats.
 * Gre l'envoi to, cc et bcc et permet d'envoyer des PJ.
 * Normalement conforme aux RFC 2045, 2046, 2047, 2110, 2111, 2183, 2387, 2822
 *
 */
class kws_mail
{
	/**
	 * Format du mail
	 *
	 * @var integer
	 *	- MAIL_TXT	: texte brut (valeur par dfaut)
	 *	- MAIL_HTML : html
	 *	- MAIL_MULTI: Multi-format (html ou texte si html non support)
	 *
	 * @access public
	 */
	var $format = MAIL_TXT;

	/**
	 * Les infos sur l'origine du mail
	 *
	 * @var		array
	 * @access	private
	 */
	var $origine = array(
		'ip'		=> '',
		'serveur'	=> ''
	);

	/**
	 * La mthode utilise par la fonction mail() pour envoyer les messages.
	 *	Gnralement sendmail sur serveur UNIX et SMTP sur serveur Windows
	 *	Selon cette methode, l'utilisation des adresses personalises diffre !!
	 *
	 * @var		string
	 * @access	private
	 */
	var $mail_methode = 'inconnu';

	/**
	 * Liste des adresses
	 *
	 * @var		array
	 * @access	private
	 */
	var $adresses = array(
		'From'		=> array(),
		'Reply-To'	=> array(),
		'To'		=> array(),
		'Cc'		=> array(),
		'Bcc'		=> array()
	);

	/**
	 * Sujet du mail
	 *
	 * @var		string
	 * @access	public
	 */
	var $sujet = '';

	/**
	 * Message du mail
	 *
	 * @var		array
	 * @access	public
	 */
	var $message = array();

	/**
	 * Pieces jointes au mail
	 *
	 * @var		array
	 * @access	public
	 */
	var $pieceJointe = array();

	/**
	 * "Frontires" utilises pour sparer les diffrentes parties du mail
	 *
	 * @var		array
	 * @access	private
	 */
	var $frontiere = array();

	/**
	 * Tableau des en-ttes complmentaires du mail
	 *
	 * @var		array
	 * @access	private
	 */
	var $headers = array();

	/**
	 * Methode d'envoi des mails en copie cache
	 *
	 * @var	 int	- MAIL_BCC : adresse en Bcc via le header (valeur par defaut)
	 *				- MAIL_TO1 : envoyer un mail par destinataire
	 *				- MAIL_TO2 : toutes les adresses en To (toutes les adresses sont visibles par tous les destinataires !!)
	 * @access	private
	 */
	var $copies_cachees = MAIL_BCC;

	/**
	 * Jeu de caractres pour le mail
	 *
	 * @var		string
	 * @access	public
	 */
	var $charset = 'iso-8859-15';

	/**
	 * Encodage  utiliser
	 *
	 * @var		string (Valeurs possible : 7bit, 8bit, quoted-printable, base64 ou binary)
	 * @access	public
	 */
	var $encodage = '8bit';

	/**
	 * Longueur maximale des lignes du mail.
	 * Dfini  78 par la RFC-2822 2.1.1 il ne devrait normalement pas tre modifi !
	 *
	 * @var		int
	 * @access	private
	 */
	var $longeurLigne = 78;

	/**
	 * Message d'erreur
	 *
	 * @var		string
	 * @access	public
	 */
	var $erreur_msg = '';

	/**
	 * Affichage des infos de debuggage
	 *
	 * @var		bool
	 * @access	public
	 */
	var $debug = false;

	/**
	 * cration d'un fichier log pour debuggage
	 *
	 * @var		bool
	 * @access	public
	 */
	var $log = false;

	/**
	 * Utilis pour dbuggage uniquement.
	 * Si valeur false, le mail ne sera pas envoy !
	 *
	 * @var		bool
	 * @access	public
	 */
	var $envoyer = true;

// ==================================================================================================
// 									LES METHODES PUBLIQUES
// ==================================================================================================

	/**
	 * Constructeur
	 *
	 * @access public
	 * @return void
	 */
//	function kws_mail($IpServ='')
	function __construct($IpServ='')
	{
		// personnalisation de paramtres
		global $K_email_options;
		if (!is_array($K_email_options))	$K_email_options = unserialize($K_email_options);
		if($K_email_options['debug']==1)	$this->debug	= true;
		if($K_email_options['log']==1)		$this->log		= true;
		if($K_email_options['bloquer']==1)	$this->envoyer	= false;

		// initialisation donnes sur l'origine du mail
		if (!empty($_SERVER['SERVER_NAME']))
		{
			$this->origine['serveur'] = $_SERVER['SERVER_NAME'];
			if (substr($_SERVER['SERVER_NAME'], -8)=='.free.fr') 
			{
				$this->isFree			= true;
				$this->freeTentatives	= (!isset($K_email_options['free_tentatives']) || $K_email_options['free_tentatives']<1 || $K_email_options['free_tentatives']>10) ? 5 : $K_email_options['free_tentatives'];
				$this->freeTpsRef		= (!isset($K_email_options['free_tpsRef']) || $K_email_options['free_tpsRef']<1 || $K_email_options['free_tpsRef']>10) ? 1.5 : $K_email_options['free_tpsRef'];
				$this->freePause		= (!isset($K_email_options['free_pause']) || $K_email_options['free_pause']<1 || $K_email_options['free_pause']>5) ? 2 : $K_email_options['free_pause'];
			}
		}
		else
		{
			$this->origine['serveur'] = 'Inconnu';
		}

		if (!empty($IpServ))
		{
			$this->origine['ip'] = $_SERVER['SERVER_ADDR'];
		}
		elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR']))
		{
			$this->origine['ip'] = $_SERVER['HTTP_X_FORWARDED_FOR'];
		}
		elseif(!empty($_SERVER['HTTP_CLIENT_IP']))
		{
			$this->origine['ip'] = $_SERVER['HTTP_CLIENT_IP'];
		}
		else
		{
			$this->origine['ip'] = $_SERVER['REMOTE_ADDR'];
		}

		// dtermination mthode d'envoi des messages par mail()
		if (@ini_get('sendmail_path') != '')
		{
			$this->mail_methode = 'sendmail';
		}
		elseif (@ini_get('sendmail_from') != '')
		{
			$this->mail_methode = 'smtp';
		}

		// initialisation frontires
		$this->frontiere[0] = '--=_Kws0_'.md5(uniqid(rand())).'_Kws0_=';
		$this->frontiere[1] = '--=_Kws1_'.md5(uniqid(rand())).'_Kws1_=';
		$this->frontiere[2] = '--=_Kws2_'.md5(uniqid(rand())).'_Kws2_=';

		// gestion des mails cachs
		global $K_mail_caches;
		if (isset($K_mail_caches) && ($K_mail_caches==MAIL_TO1 || $K_mail_caches==MAIL_TO2))
		{
			$this->copies_cachees = $K_mail_caches;
		}
	}


	/**
	 * Dfinition du format du mail  envoyer
	 *
	 * @access	public
	 * @param	mixed $format  Format de l'email
	 *	- 0, txt ou texte	: mail texte
	 *	- 1, htm, html		: mail html
	 *	- 2, multi, alt		: mail multiformat
	 * @return	void
	 */
	function set_format($format)
	{
		$format = strtolower($format);

		switch($format)
		{
			case 2:
			case 'multi':
			case 'alt':
				$this->format = MAIL_MULTI;
			break;

			case 1:
			case 'htm':
			case 'html':
				$this->format = MAIL_HTML;
			break;

			case 0:
			case 'texte':
			case 'txt':
			default:
				$this->format = MAIL_TXT;
		}
	}


	/**
	 * Dfinition du jeu de caractre
	 *
	 * @access	public
	 * @param	string	$charset
	 * @return	void
	 */
	function set_charset($charset)
	{
		$this->charset = $charset;
	}


	/**
	 * Vrification la validit syntaxique d'une adresse email
	 *
	 * @access	public
	 * @param	string 	$adresse	adresse mail  vrifier
	 * @return	bool
	 */
	function validerAdresse($adresse)
	{
		$atom	= '[-a-z0-9!#$%&"*+\\/=?^_`{|}~]';		// caractres autoriss avant l'arobase
		$domain = '([a-z0-9]([-a-z0-9]*[a-z0-9]+)?)';	// caractres autoriss aprs l'arobase (nom de domaine)

		$regex = '/^' . $atom . '+' .					// Une ou plusieurs fois les caractres autoriss avant l'arobase
		'(\.' . $atom . '+)*' . 						// Suivis par zro point ou plus
														// spars par des caractres autoriss avant l'arobase
		'@' .											// Suivis d'un arobase
		'(' . $domain . '{1,63}\.)+' .					// Suivis par 1  63 caractres autoriss pour le nom de domaine
														// spars par des points
		$domain . '{2,63}$/i';							// Suivi de 2  63 caractres autoriss pour le nom de domaine

		return (bool) preg_match($regex, $adresse);
	}


	/**
	 * Ajout d'adresse(s)

	 * @access	public
	 * @param	mixed	$adresses	adresse(s)  ajouter
	 * 	accepte :
	 *		- les adresses type addr-spec ( adresse@host.ext - Cf. RFC-2822)
	 *		- les adresses type name-addr ( [nom] <adresse@host.ext> - Cf. RFC-2822)
	 *		- les listes d'adresses addr-spec ( array('adresse@host.ext', 'adresse@host.ext', 'adresse@host.ext', ...) )
	 *		- les listes d'adresses name-addr ( array('nom' => 'adresse@host.ext', 'nom' => 'adresse@host.ext', ...) )
	 * @param	string	$type	Type de adresses
	 * 	accepte (insensible  la casse) :
	 *		- From, De
	 *		- Reply-To, reply to, repondre-a, repondre a
	 *		- To, A (defaut)
	 *		- Cc
	 *		- Bcc, Cci
	 * @return	bool :
	 *	- true si tout se passe bien,
	 *	- false si au moins une adresse est rejete. Dtails dans $this->erreur_msg
	 */
	function add_adresses($adresses, $type='To')
	{
		$type = strtolower($type);
		static $arrayType = array(
			'from'		=> 'From',
			'de'		=> 'From',
			'a'			=> 'To',
			'cc'		=> 'Cc',
			'bcc'		=> 'Bcc',
			'cci'		=> 'Bcc',
			'reply-to'	=> 'Reply-To',
			'reply to'	=> 'Reply-To',
			'repondre-a'=> 'Reply-To',
			'repondre a'=> 'Reply-To'
		);
		if (array_key_exists($type, $arrayType))
		{
			$type = $arrayType[$type];
		}
		else
		{
			$type = 'To';
		}

		// si adresse unique, on la met sous forme de tableau
		if (!is_array($adresses))
		{
			// adresse type name-addr (cf. RFC-2822)
			if (preg_match('`^(.*)<(.+)>$`', trim($adresses), $matches))
			{
				list(,$nom, $mail) = $matches;
				if (!empty($nom))
				{
					$adresses = array($nom => $mail);
				}
				else
				{
					$adresses = array($mail);
				}
			}
			// autres formats d'adresse
			else
			{
				$adresses = array($adresses);
			}
		}

		// on traite chaque adresse
		$this->erreur_msg = '';
		foreach ($adresses as $nom => $mail)
		{
			$mail = trim($mail);
			if (!($this->validerAdresse($mail) || (empty($mail) && $type=='To' && !empty($nom))))
			{
				$this->erreur_msg .= 'Adresse '.$type.' ('.$mail.') non valide, donc rejete !<br />';
				continue;
			}

			$this->adresses[$type][] = array(
				'nom' 	=> $this->_encoder_entete($type, $nom, 'phrase'),
				'mail'	=> $this->_proteger_entete($mail)
			);
		}

		return empty($this->erreur_msg);
	}


	/**
	 * Dfinition du sujet du mail
	 *
	 * @access	public
	 * @param	string	$sujet	sujet du mail
	 * @return	void
	 */
	function set_sujet($sujet)
	{
		$this->sujet = $this->_encoder_entete('Subject', trim(strip_tags($sujet)));
	}


	/**
	 * Dfinition du corps du message
	 * Un message pourra tre purement html, purement texte ou mixte.
	 * Dans ce dernier cas le message alternatif correspondra au message html propos
	 * sans les balises html
	 *
	 * @access	public
	 * @param	string	$message	comme son nom l'indique ;)
	 * @param	string	$format		format du message : html ou texte ou multiformat
	 * @return	void
	 */
	function set_message($message, $format=MAIL_TXT)
	{
		global $K_email_habillage;
		if (preg_match('#%contenumail%#',$K_email_habillage))
		{
			$message = str_replace('%contenumail%', $message, $K_email_habillage);
		}
		// On vire les anti-slashes ajouts par les magic_quotes de m.... !
		if (get_magic_quotes_gpc())
		{
			$message = stripslashes($message);
		}

		$this->format = MAIL_TXT;
		switch (strtolower($format))
		{
			case MAIL_HTML:
			case 'html':
			case 'htm':
				$this->message['html'] = $message;
				$this->format = MAIL_HTML;
			break;

			case MAIL_MULTI:
			case 'multi':
			case 'alt':
				$this->message['html'] = $message;
				$this->format = MAIL_MULTI;
			case MAIL_TXT:
			case 'txt':
			case 'texte':
			default:
				$this->message['txt'] = $this->_html2txt($message);
		}
	}


	/**
	 * Dfinition de la priorit du mail
	 *
	 * @access	public
	 * @param	mixed	$priorite	Niveau de priorit du mail
	 *	- de 1  5 ou de Highest  lowest. Par dfaut : 3 - Normal
	 * @return	void
	 */
	function set_priorite($priorite)
	{
		switch (strtolower($priorite))
		{
			case '1':
			case 'highest':
				$this->headers['X-Priority'] = '1';
				$this->headers['X-MSMail-Priority'] = 'Highest';
			break;

			case '2':
			case 'high':
				$this->headers['X-Priority'] = '2';
				$this->headers['X-MSMail-Priority'] = 'High';
			break;

			case '4':
			case 'low':
				$this->headers['X-Priority'] = '4';
				$this->headers['X-MSMail-Priority'] = 'Low';
			break;

			case '5':
			case 'lowest':
				$this->headers['X-Priority'] = '5';
				$this->headers['X-MSMail-Priority'] = 'Lowest';
			break;

			default:
				$this->headers['X-Priority'] = '3';
				$this->headers['X-MSMail-Priority'] = 'Normal';
			break;
		}
	}


	/**
	 * Prise en compte d'ventuelles en-ttes complmentaires
	 *
	 * @access	public
	 * @param	string	$champ   Nom de l'en-tte
	 * @param	string	$valeur  Contenu de l'en-tte
	 * @return	bool
	 */
	function add_header($champ, $valeur)
	{
		// Vrif conformit  la norme (RFC2822 2.2)
		if (!preg_match('`^[!-9;-~]+$`', $champ))
		{
			$this->erreur_msg = 'champ ('.$champ.') non valide !';
			return false;
		}
		$valeur = preg_replace('`[\x0A\x0D]`', '', $valeur);

		$this->headers[$champ] = $valeur;

		return true;
	}


	/**
	 * Ajout d'une pice jointe
	 *
	 * @access	public
	 * @param	string	$fichier		Nom du fichier (y compris chemin d'acces)
	 * @param	string	$disposition	Disposition : inline ou attachement(par dfaut)
	 * @param	bool	$inclure		true si on doit l'inclure dans un mail html
	 * @return	bool
	 */
	function add_pieceJointe($fichier, $disposition='attachement', $inclure=false)
	{
		// validit du fichier
		if (!is_readable($fichier))
		{
			$this->erreur_msg = 'La pice jointe '.$fichier.' est introuvable ou inaccessible !';
			return false;
		}

		array_push($this->pieceJointe, array(
			'fichier'		=> $fichier,
			'disposition'	=> strtolower($disposition)=='inline' ? 'inline' : 'attachement',
			'inclure'		=> (bool) $inclure
		));

		return true;
	}


	/**
	 * gestion de l'envoi du mail
	 *
	 * @access	public
	 * @return	bool
	 */
	function envoyer($renvoiFree=false)
	{
		global $K_IN_DEMO;
		$DemoMsg = '';
		if ($K_IN_DEMO)
		{
			$this->log = false;
			$this->envoyer = false;
			$DemoMsg = 'Vous tes sur le site de dmonstration.<br />';
		}

		// si copies (Cc ou Bcc) non autorises
		if ($this->copies_cachees != MAIL_BCC)
		{
			// On transfert toutes les adresses en To
			$this->adresses['To']	= array_merge($this->adresses['To'], $this->adresses['Cc'], $this->adresses['Bcc']);
			$this->adresses['Cc']	= array();
			$this->adresses['Bcc']	= array();
		}

		do
		{
			if (($destinataire	= $this->_get_destinataires())	=== false ||
				($message		= $this->_get_message())		=== false ||
				($entetes		= $this->_get_entetes())		=== false)
			{
				return false;
			}
			$sujet = empty($this->sujet) ? 'Sans sujet' : $this->sujet;

			if ($this->debug)
			{
				echo '<p><u>Envoi du message</u> :<br /><br />
				<b>destinataire : </b>'.nl2br(htmlspecialchars($destinataire)).'<br /><br />
				<b>sujet : </b>'.nl2br(htmlspecialchars($sujet)).'<br /><br />
				<b>message : </b>'.nl2br(htmlspecialchars($message)).'<br /><br />
				<b>en-ttes : </b>'.nl2br(htmlspecialchars($entetes)).'</p>';
			}

			if ($this->envoyer)
			{
				$tentative	= 0;
				$envoi		= false;
				do
				{
					$tentative++;
					$t0		= microtime(TRUE);
					$result	= @mail($destinataire, $sujet, $message, $entetes);
					$t1		= microtime(TRUE);
					$duree	= $t1 - $t0;
					if ($this->isFree && (!$result || $duree < $this->freeTpsRef))
					{
						// chec de cette tentative
						$result = false;
						$this->erreur_msg = 'mail Free :: tentative '.$tentative.' - dure : '.round($duree, 4).' => chec de l\'envoi';
						if ($this->log)	$this->_log($destinataire, $sujet, $entetes, $result);
						$pause = 0;
						// pause entre 2 tentatives (Ne pas descendre plus bas que 2s : moins bons rsultats)
						while ($pause < $this->freePause)
						{ 
							$pause = microtime(TRUE) - $t1;
						}
					}
					elseif (!$this->isFree && !$result)
					{
						$this->erreur_msg = 'Une erreur s\'est produite lors de l\'envoi du mail !';
					}
					else $envoi = true;

				} While ($this->isFree && $envoi == false && $tentative < $this->freeTentatives); // X tentatives autorises pour free
				
				if ($this->isFree && $envoi == false)
				{
					// Echec envoi des X tentatives free 
					$this->erreur_msg = 'le mail n\'a pu tre envoy malgr '.$tentative.' tentatives. Il a t mmoris pour tre renvoy dans quelques minutes...';

					// Ajout du N de renvoi dans le sujet (RVxx :)
					if (preg_match('`RV([0-9]+) :`', $sujet, $matches))		$titre = str_replace($matches[1], ++$matches[1], $sujet);
					else 													$titre = 'RV1 : '.$sujet;

					// Enregistrement pour renvoi ultrieur
					$to = serialize($this->adresses);
					$mess = serialize($this->message);
					reqmysql('INSERT INTO	`emails_to_send`
								SET			`type` = "'.sp($this->format).'",
											`titre` = "'.sp($titre).'",
											`message` = "'.sp($mess).'",
											`from` = "",
											`to` = "'.sp($to).'",
											`cc` = "",
											`cci` = "",
											`module` = "Mail_Free"
					;');

					global $K_email_check;
					if ($K_email_check['stand_by'] == 99999999999)
					{
						$K_email_check['stand_by'] = 0;
						reqmysql('UPDATE `parametres` SET `valeur`="'.sp(serialize($K_email_check)).'" WHERE `nom` = "K_email_check";');
					}

					if (!$renvoiFree)
					{
						$msg = df_message('attention', $titre, 'Echec de l\'envoi ! Une nouvelle tentative aura lieu plus tard.');
						reqmysql('INSERT INTO `alerte` SET `date`="'.time().'", `auteur`="Portail", `module`="Envoi mails Free.fr", `texte`="'.sp($msg).'";');
					}
				}
			}
			else
			{
				$this->erreur_msg = $DemoMsg.'Le mail n\'a pas t envoy !';
				$result = false;
			}

			if ($this->log)		$this->_log($destinataire, $sujet, $entetes, $result, '('.$duree.')');

		// Si envoi en destinataire unique, on boucle tant qu'il reste des destinataires.
		} while ($this->copies_cachees == MAIL_TO1 && !empty($this->adresses['To']));

		return $result;
	}


	/**
	 * Enregistrement des log
	 *
	 * @access	public
	 * @return	void
	 */
	function _log($destinataire, $sujet, $entetes, $result, $duree='')
	{
		$log = '
Date : '.Kws_date('d-m-Y H:i', time()).'
Mthode : '.$this->mail_methode.'
Copies caches : '.$this->copies_cachees.'
Destinataire : '.$destinataire.'
Sujet : '.$sujet.'
Entte : '.$entetes.'
Rsultat : '.($result ? 'Envoi OK '.$duree : $this->erreur_msg).'
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

';
		$f = fopen('./emails.log', 'ab+');
		fwrite($f, $log);
		fclose($f);
	}
		

	/**
	 * Rinitialisation
	 *
	 * @access	public
	 * @return	void
	 */
	function raz($adresses=array(), $sujet=0, $message=0, $pj=0, $headers=0)
	{
		if (is_array($adresses))
		{
			foreach ($adresses as $dest)
			{
				$this->adresses[$dest] = array();
			}
		}

		if ($sujet)
		{
			$this->sujet = '';
		}

		if ($message)
		{
			$this->message = array();
		}

		if ($pj)
		{
			$this->pieceJointe = array();
		}

		if ($headers)
		{
			$this->headers = array();
		}
	}


// ==================================================================================================
// 									LES METHODES PRIVEES
// ==================================================================================================

	/**
	 * Gnre la liste des destinataires ('To')
	 *
	 * @access private
	 * @return mixed
	 */
	function _get_destinataires()
	{
		// On s'assure qu'ily a au moins un destinataire !
		if (empty($this->adresses['To']) && empty($this->adresses['Cc']))
		{
			if (empty($this->adresses['Bcc']))
			{
				$this->erreur_msg = 'Votre mail ne contient aucun destinataire !';
				return false;
			}
			$this->adresses['To'][] = array(
				'nom'	=> $this->_encoder_entete('To', 'Destinataires non rvls:;', 'phrase'),
				'mail'	=> ''
			);
		}

		$destinataire = '';
		if (!empty($this->adresses['To']))
		{
			foreach ($this->adresses['To'] as $i => $adresse)
			{
				$destinataire .= !empty($destinataire) ? ', ' : '';
				if (!empty($adresse['nom']) && $this->mail_methode=='sendmail')
				{
					// Utilisation des adresses personnalises pour sendmail uniquement
					$destinataire .= $adresse['nom'].' <'.$adresse['mail'].'>';
				}
				else
				{
					$destinataire .= $adresse['mail'];
				}

				// Si envoi en destinataire unique, on sort de la boucle
				if ($this->copies_cachees == MAIL_TO1)	break;
			}
		}
		if ($this->debug)
		{
			echo '<p><b>Destinataire(s)</b> : '.$destinataire.'</p>';
		}
		return $destinataire;
	}


	/**
	 * Gnre les en-ttes
	 *
	 * @access	private
	 * @return	mixed
	 */
	function _get_entetes()
	{
		// L'expditeur (From) est obligatoire et unique !
		if (count($this->adresses['From']) != 1)
		{
			$this->erreur_msg = 'Erreur d\'Expditeur. Abandon du traitement !';
			return false;
		}

		$entete = '';
		foreach (array('From', 'To', 'Cc', 'Bcc', 'Reply-To') as $addr)
		{
			// Ne pas mettre les adresses To dans le header si sendmail,
			// sinon message en double !
			if ($addr=='To' && $this->mail_methode=='sendmail')
			{
				// Si envoi en destinataire unique,
				// il faut quand mme virer la premire adresse de la liste To
				// car on l'a dj prise en compte dans les destinataires.
				if ($this->copies_cachees == MAIL_TO1)	array_shift($this->adresses[$addr]);
				continue;
			}

			if (count($this->adresses[$addr])>0)
			{
				$entetes[$addr] = '';
				foreach ($this->adresses[$addr] as $adresse)
				{
					$entetes[$addr] .= !empty($entetes[$addr]) ? ', ' : '';
					if (!empty($adresse['nom']) && ($this->mail_methode=='sendmail' || $addr=='From'))
					{
						// Utilisation des adresses personnalises :
						//  - pas de pb pour sendmail,
						//  - uniquement pour le champ From dans les autres cas.
						$entetes[$addr] .= $adresse['nom'].' <'.$adresse['mail'].'>';
					}
					else
					{
						$entetes[$addr] .= $adresse['mail'];
					}

					// Si envoi en destinataire unique,
					// on supprime l'adresse des destinataires et on sort de la boucle
					if ($addr=='To' && $this->copies_cachees == MAIL_TO1)
					{
						array_shift($this->adresses[$addr]);
						break;
					}
				}
				// pb d'entete avec free si on coupe la chaine d'adresse
				if (strpos($this->origine['serveur'], 'free.fr')===false)
				{
					$entete .= wordwrap($addr.': '.$entetes[$addr], $this->longeurLigne, "\n ")."\n";
				}
				else
				{
					$entete .= $addr.': '.$entetes[$addr]."\n";
				}
			}
		}

		// si Reply-To n'est pas prcis, on remplis avec adresse From
		if (strpos($entete, 'Reply-To')===false)
		{
			$entete .= wordwrap('Reply-To: '.$entetes['From'], $this->longeurLigne, "\n ")."\n";
		}

		$entete	.= 	wordwrap('Date: '.date('r', time()), $this->longeurLigne, "\n ")."\n".
					"MIME-Version: 1.0\n".
					"X-Mailer: KwsPHP\n".
					wordwrap('X-AntiAbuse: Sender IP - '.$this->origine['ip'].'/Server Name - <'.$this->origine['serveur'].'>', $this->longeurLigne, "\n ")."\n".
					wordwrap('Message-ID: <'.md5(microtime()).'@'.$this->origine['serveur'].'>', $this->longeurLigne, "\n ")."\n";

		foreach ($this->headers as $nom => $valeur)
		{
			if (!empty($valeur))
			{
				$entete .= wordwrap($nom.': '.$valeur, $this->longeurLigne, "\n ")."\n";
			}
		}

		return $entete.$this->_content;
	}


	/**
	 * Gnre le message proprement dit
	 *
	 * @access	private
	 * @return	mixed
	 */
	function _get_message()
	{
		// Controle d'intgrit
		if (empty($this->message) && empty($this->pieceJointe))
		{
			$this->erreur_msg = 'Il n\'y a ni message ni pice jointe  envoyer ! Procdure abandonne !';
			return false;
		}
		if (!empty($this->message['html']) && !empty($this->message['txt']))
		{
			$this->format = MAIL_MULTI;
		}
		if ($this->format==MAIL_MULTI && empty($this->message['txt']))
		{
			$this->message['txt'] = $this->_html2txt($this->message['html']);
		}
		if (($this->format==MAIL_MULTI || $this->format==MAIL_HTML) && empty($this->message['html']))
		{
			$this->format = MAIL_TXT;
		}

		if ($this->format == MAIL_MULTI)
		{
			// Message alternatif (html et txt)
			$PJ = '';
			if (empty($this->pieceJointe))
			{
				$this->_content = 'Content-Type: multipart/alternative; boundary="'.$this->frontiere[1].'"';
				$message = 'This is a multi-part message in MIME format'."\n\n";
			}
			else
			{
				$this->_content = 'Content-Type: multipart/related; boundary="'.$this->frontiere[0].'"';

				$message =
					'This is a multi-part message in MIME format'."\n\n".
					'--'.$this->frontiere[0]."\n".
					'Content-Type: multipart/alternative; boundary="'.$this->frontiere[1].'"'."\n\n";

				foreach ($this->pieceJointe as $pieceJointe)
				{
					$PJ .=
						'--'.$this->frontiere[0]."\n".
						$this->_get_pj($pieceJointe);
				}

				$PJ .= '--'.$this->frontiere[0]."--\n\n";
			}

			$message .=
				'--'.$this->frontiere[1]."\n".
					'Content-Type: text/plain; charset='.$this->charset."\n".
					'Content-Transfer-Encoding: '.$this->encodage."\n\n".
					$this->_encoder_message($this->message['txt'], $this->encodage)."\n\n".
				'--'.$this->frontiere[1]."\n".
					'Content-Type: text/html; charset='.$this->charset."\n".
					'Content-Transfer-Encoding: '.$this->encodage."\n\n".
					$this->_encoder_message($this->message['html'], $this->encodage)."\n".
				'--'.$this->frontiere[1]."--\n\n".$PJ;
		}
		elseif (!empty($this->pieceJointe))
		{
			// message html ou txt avec PJ
			$this->_content = 'Content-Type: multipart/mixed; boundary="'.$this->frontiere[0].'"';

			foreach ($this->pieceJointe as $pieceJointe)
			{
				if (!empty($pieceJointe['inclure']) && $this->format==MAIL_HTML)
				{
					$this->_content = 'Content-Type: multipart/related; boundary="'.$this->frontiere[0].'"';
				}
				$PJ .=
					'--'.$this->frontiere[0]."\n".
					$this->_get_pj($pieceJointe);
			}
			$PJ .= '--'.$this->frontiere[0]."--\n\n";

			$message =
				'This is a multi-part message in MIME format'."\n\n".
				'--'.$this->frontiere[0]."\n".
					'Content-Type: text/'.($this->format==MAIL_HTML ? 'html' : 'plain').'; charset='.$this->charset."\n".
					'Content-Transfer-Encoding: '.$this->encodage."\n\n".
					$this->_encoder_message($this->message[$this->format==MAIL_HTML ? 'html' : 'txt'], $this->encodage)."\n\n".
					$PJ;
		}
		elseif ($this->format == MAIL_HTML)
		{
			// Message html sans PJ
			$this->_content = 	'Content-Type: text/html; charset='.$this->charset."\n".
								'Content-Transfer-Encoding: '.$this->encodage;
			$message = $this->_encoder_message($this->message['html'], $this->encodage);
		}
		else
		{
			// Message texte sans PJ
			$this->_content = 	'Content-Type: text/plain; charset='.$this->charset."\n".
								'Content-Transfer-Encoding: '.$this->encodage;
			$message = $this->_encoder_message($this->message['txt'], $this->encodage);
		}

		return $message;
	}


	/**
	 * Gnre le 'body-part' pour une PJ
	 *
	 * @access private
	 * @param	array	$pieceJointe	Tableau contenant tous les lements identifiant la PJ
	 * @return string
	 */
	function _get_pj($pieceJointe)
	{
		$nomFichier = basename($pieceJointe['fichier']);
		$pj =	'Content-Type: '.mime_content_type($nomFichier).'; name="'.$nomFichier."\"\n".
				'Content-Transfer-Encoding: base64'."\n";

		if ($pieceJointe['inclure'] && $this->format>0)
		{
			$cid = md5(microtime()).'@Kws';
			$this->message['html'] = str_replace('cid:'.$nomFichier, 'cid:'.$cid, $this->message['html']);
			$pj .= 'Content-ID: <'.$cid.'>'."\n\n";
		}
		else
		{
			$pj .= 'Content-Disposition: '.($this->format==0 ? 'attachement' : $pieceJointe['disposition']).'; filename="'.$nomFichier."\"\n\n";
		}

		// lecture du fichier  joindre
		$fp = fopen($pieceJointe['fichier'], 'rb');
		$fichierContenu = fread($fp, filesize($pieceJointe['fichier']));
		fclose($fp);
		$pj .= $this->_encoder_message($fichierContenu, 'base64')."\n\n";

		return $pj;
	}


	/**
	 * Encodage des lments de l'en-tte
	 *
	 * @access	private
	 * @param	string	$nom		Nom de l'en-tte correspondant
	 * @param	string	$valeur		Contenu de l'en-tte
	 * @param	string	$type		Type de donnes  encoder (Cf. RFC-2047 5)
	 *								 	- 'text' pour le contenu des champs (valeur par dfaut)
	 *								 	- 'comment' pour les commentaires
	 *								 	- 'phrase' pour le nom des champs
	 * @return	string
	 */
	function _encoder_entete($nom, $valeur, $type='text')
	{
		if ($this->debug) echo '<p>nom : '.$nom.' // valeur : '.$valeur.' // type : '.$type.'<br />';

		if (empty($valeur)) return;
		$valeur = $this->_proteger_entete($valeur);

		// Les caractres  encoder en fonction du type
		switch (strtolower($type))
		{
			case 'comment':
				$masque		= '[\x00-\x1F"()/=?_\x7F-\xFF]';
			break;

			case 'phrase':
				$masque		= '[^A-Za-z0-9 !*+/-]';
			break;

			case 'text':
			default:
				$masque		= '[\x00-\x1F=?_\x7F-\xFF]';
			break;
		}

		// Dtermination du type d'encodage  utiliser ("B" ou "Q")
		// en fonction du nombre de caractres  encoder,
		// de faon  obtenir la chaine encode la plus courte.
		$nb = preg_match_all('`'.$masque.'`', $valeur, $matches);
		if ($this->debug) echo 'nombre remplacement : '.$nb;

		if ($nb == 0)
		{
			// pas d'encodage ncessaire,
			if ($this->debug) echo ' == >> pas d\'encodage<br />';
			// ventuellement mettre la chane entre ""
			if ($type != 'text' && preg_match('/[^!#$%&\'*+\/0-9=?a-z^_`{|}~-]/', $valeur))
			{
				$encoded = '"'.$valeur.'"';
			}
			else $encoded = $valeur;
		}
		elseif (strlen($valeur)/3 > $nb)
		{
			// encodage "Q"
			if ($this->debug)
			{
				echo ' == >> encodage <b>Q</b><br /><pre>';
				print_r($matches);
				echo '</pre>';
			}

			//$Q = preg_replace('`('.$masque.')`e', "'='.sprintf('%02X', ord('$1'))", $valeur);
			$Q = preg_replace_callback('`('.$masque.')`',function ($Kmt){return '='.sprintf('%02X', ord($Kmt[1]));},$valeur);
			$Q = str_replace(' ', '_', $Q);
			// on scinde en 'mot' de 76 caractres sans couper au milieu d'une entit encode)
			$encoded = '';
			while (strlen($Q)>($longueur=68-strlen($this->charset)-strlen($nom)))
			{
				$i=2;
				while ($Q[($longueur-$i)] != '=' && $i>0) $i--;
				$encoded .= '=?'.$this->charset.'?Q?'.substr($Q, 0, $longueur-$i)."?=\n "; // \r
				$Q = trim(substr($Q, $longueur-$i));
				$nom = '';
			}
			$encoded .= '=?'.$this->charset.'?Q?'.$Q."?= ";
		}
		else
		{
			// encodage "B"
			if ($this->debug)
			{
				echo ' == >> encodage <b>B</b><br /><pre>';
				print_r($matches);
				echo '</pre>';
			}

			// on encode par tronon multiple de 4 sans dpasser la longeur max de 76 caractres tout compris
			$longueur = 68-strlen($this->charset)-strlen($nom);
			$longueur -= $longueur%4;
			$encoded  = trim(chunk_split(base64_encode($valeur), $longueur, "\n"));
			$encoded  = '=?'.$this->charset.'?B?'.str_replace("\n", "?=\n =?".$this->charset.'?B?', $encoded)."?=\n";
			//$encoded  = str_replace($nom.': ', '', $encoded);
		}

		if ($this->debug) echo 'encod : '.$encoded.'</p>';
		return $encoded;
	}


	/**
	 * Encodage du message proprement dit
	 *
	 * @access	private
	 * @param	string	$encodage	Type d'encodage dsir
	 * @param	string	$texte		Texte  encoder
	 * @return	string
	 */
	function _encoder_message($texte, $encodage)
	{
		switch (strtolower($encodage))
		{
			case 'base64':
				// ref : RFC2045 6.8
				$texte = rtrim(chunk_split(base64_encode($texte), 76, "\n"));
			break;

			case 'quoted-printable':
				// ref : RFC2045 6.7
				$texte = preg_replace('`\r\n?`', "\n", $texte);
				// encodage caractres CTL, = et >127
				//$texte = preg_replace('`([\x00-\x08\x0B\x0C\x0E-\x1F\x3D\x7F-\xFF])`e', "'='.sprintf('%02X', ord('$1'))", $texte);
				$texte = preg_replace_callback('`([\x00-\x08\x0B\x0C\x0E-\x1F\x3D\x7F-\xFF])`', function ($Kmt){return '='.sprintf('%02X', ord($Kmt[1]));}, $texte);
				// encodage tab et espace de fin de ligne
				//$texte = preg_replace('`([\x09\x20])\n`e', "'='.sprintf('%02X', ord('$1')).\"\n\"", $texte);
				$texte = preg_replace_callback('`([\x09\x20])\n`',  function ($Kmt){return '='.sprintf('%02X', ord($Kmt[1]))."\n";}, $texte);
				// On coupe en 'mots' de 76 caractres se terminant par un =
				$tmp = '';
				while (strlen($texte)>74)
				{
					$i=2;
					while ($texte[(74-$i)] != '=' && $i>0) $i--;
					$tmp .= substr($texte, 0, 74-$i)."=\n"; // \r
					$texte = trim(substr($texte, 74-$i));
				}
				$texte = $tmp.$texte;
			break;

			case 'binary':
				// ref : RFC2045 2.9
			break;

			case '7bit':
			case '8bit':
			default:
				// ref : RFC2045 2.7 et 2.8
				$texte = preg_replace("`\r\n?`", "\n", $texte);
				$tmp = explode("\n", $texte);
				$texte = '';
				foreach ($tmp as $l)
				{
					$texte .= wordwrap($l, $this->longeurLigne, "\n")."\n";
				}
		}

		return trim($texte);
	}


	/**
	 * Protection contre les injection d'en-tte
	 * Supprime tous les retours  la ligne qui pourraient permettre
	 * une injection d'header dans le mail.
	 *
	 * @acces	private
	 * @param	string	$chaine	chaine  protger
	 * @return	string
	 */
	 function _proteger_entete($chaine)
	 {
	 	return preg_replace("[\r\n]", '', trim($chaine));
	 }


	/**
	 * Converti un texte HTML en texte brut
	 *
	 * @access	private
	 * @param	string	$texte		Texte  convertir
	 * @return	string
	 */
	function _html2txt($texte)
	{
		//$message = preg_replace('#<a(.*?)href="(.[^"\'()<>\]\[]*?)"(.*?)>(.+?)</a>#i','$4: $2',$message);

		// on recherche ...
		$replace_from = array(
			'`&amp;`',
			'`&#039;`',
			'`&euro;`',
			'`[\\r\\n]+\s*`',					// retour chariot et retour ligne non significatifs
			'`<br\s*/?\s*>\s*`i',				// passage  la ligne
			'`<hr\s*/?\s*>\s*`i',				// ligne horizontale
			'`</?(strong|b)>`i',				// gras
			'`</?(em|i)>`i',					// italic
			'`</?u>`i',							// soulign
			'`<a.* href="(.+?)".*>(.*?)</a>`i'	// liens
		);
		// pour remplacer par ...
		$replace_to = array(
			'&',
			"'",
			'',
			'',									// supprims
			"\n",								// retour ligne
			"\n\n-------------------------\n\n",// ligne horizontale
			'*',								// *gras*
			'/',								// /italic/
			'_',								// _soulign_
			' $2 ($1)'							// texte lien (lien)
		);
		$texte = preg_replace($replace_from, $replace_to, $texte);
		$texte = html_entity_decode($texte);
		return strip_tags($texte);
	}
}

?>